import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:http/http.dart' as http;
import 'package:wowit/common.dart';
import 'package:wowit/methods.dart';
import 'package:wowit/widgets/log_widget/log_controller.dart';

abstract class ISQLService {
  Future<Map<String, List<Map<String, Object>>>> execute(
    String query, {
    Map<String, Object?>? params,
  });
  Future logout();
  Future createAccountIfNotExists(String phoneNumber, String uidPhone);
}

class SQLService {
  const SQLService._();

  static ISQLService? iSQLService;

  static Future createAccountIfNotExists(
      String phoneNumber, String uidPhone) async {
    if (iSQLService != null) {
      await iSQLService?.createAccountIfNotExists(phoneNumber, uidPhone);
    } else {
      await http.post(Methods.getURI('/data/CreateAccount'), body: {
        'phoneNumber': phoneNumber,
        'uidPhone': uidPhone,
      });
    }
  }

  static Future<Map<String, List<Map<String, Object>>>> execute(
    String query, {
    Map<String, Object?>? params,
  }) async {
    if (iSQLService != null) {
      return await iSQLService?.execute(query, params: params) ?? {};
    }
    params = setupDefaultParameter(query, params);

    String jparamsString = jsonEncode(params);
    var queryParameters = {
      'query': query,
      'jParam': jparamsString,
    };
    if (kDebugMode) {
      print('Execute query: $query, jParam  $jparamsString');
    }
    Map<
        String, // table namme
        List< // list row -> data tabl
            Map<String, Object> //data row
            >> retVal = {};
    try {
      var response =
          await http.post(Methods.getURI('/data/exec'), body: queryParameters);
      if (response.statusCode == 200) {
        if (response.body == '') {
          String logStatus = ' $query queryParameters $queryParameters response.body is empty';
          LogController.setLog(log: logStatus, titleLog: 'SQLService.execute');
          if (kDebugMode) {
            print(
                '[EMPTY RESPONSE]: $logStatus');
          }
          return retVal;
        }
        var _body = response.body;

        Map<String, dynamic> jsonData =
            json.decode(_body); //--> return data set

        jsonData.forEach((key, value) {
          List<dynamic> table = value;
          List<Map<String, Object>> rsTable = [];
          for (int i = 0; i < table.length; i++) {
            rsTable.add(table[i]);
          }
          retVal[key] = rsTable;
        });
        if (kDebugMode) {
          print(
              'execute query: $query, result length: ${retVal.length.toString()}');
        }
      } else {
        String log = 'SQLService.execute with $query response status code = ${response.statusCode.toString()} ';
        LogController.setLog(log: log, titleLog: 'SQLService.execute');
        if (kDebugMode) {
          print(log);
        }
      }
    } catch (e) {
      LogController.setLog(log: e.toString(), titleLog: 'SQLService.execute');
      if (kDebugMode) {
        print('SQLService.execute with exception:  ' + e.toString());
      }
    }

    if (retVal.isNotEmpty && retVal.containsKey('Exception')) {
      var dataException = Methods.getList(retVal, 'Exception');
      if (dataException.isNotEmpty && dataException[0].containsKey('Message')) {
        var exception = Methods.getString(dataException[0], 'Message');
        if (exception != '') {
          if (kDebugMode) {
            print(
                '==================>[Exception]<==================\n==================> Query:' +
                    query +
                    '\n==================> Parameter:' +
                    params.toString() +
                    '\n==================> Message:' +
                    exception.toString());
          }
        }
      }
    }

    return retVal;
  }

  static Map<String, Object?> setupDefaultParameter(
    String query,
    Map<String, Object?>? params,
  ) {
    params ??= {};
    params['query'] = query;
    if (!params.containsKey('uid') && !params.containsKey('@uid')) {
      params['uid'] = Common.uid;
    }

    if (kDebugMode) {
      if (!params.containsKey('kDebugMode') &&
          !params.containsKey('@kDebugMode')) {
        params['kDebugMode'] = kDebugMode;
      }
    }

    if (!params.containsKey('languageId') &&
        !params.containsKey('@languageId')) {
      params['languageId'] = Common.languageId;
    }

    if (!params.containsKey('deviceName') &&
        !params.containsKey('@deviceName')) {
      params['deviceName'] = Common.deviceName;
    }
    if (!params.containsKey('@deviceId') && !params.containsKey('deviceId')) {
      params['deviceId'] = Common.deviceId;
    }

    if (!params.containsKey('@mode') && !params.containsKey('mode')) {
      params['mode'] = 'App';
    }

    if (!params.containsKey('@fVersion') &&
        !params.containsKey('fVersion') &&
        Common.fVersion > 0) {
      params['fVersion'] = Common.fVersion;
    }

    return params;
  }

  static void logView({
    String query = 'spLogView',
    required int viewID,
    required int dataID,
  }) {
    SQLService.execute(query, params: {
      'userID': Common.uid,
      'idView': viewID,
      'idData': dataID,
    });
  }

  static void logLike({
    String query = 'spLogLike',
    required int idLoai,
    required int idItem,
    required bool isLike,
  }) {
    SQLService.execute(
      query,
      params: {
        'userID': Common.uid,
        'idView': idLoai,
        'idData': idItem,
        'isLike': isLike
      },
    );
  }

  static void logFollow({
    String query = 'spLogFollow',
    required int idLoai,
    required int idItem,
    required bool isFollow,
  }) {
    SQLService.execute(query, params: {
      'userID': Common.uid,
      'idView': idLoai,
      'idData': idItem,
      'isFollow': isFollow
    });
  }
}
