import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:wowit/controllers/base_controller.dart';
import 'package:wowit/controllers/filter_controller.dart';
import 'package:wowit/controllers/load_more_base_controller.dart';
import 'package:wowit/controllers/searched_controller.dart';
import 'package:wowit/message.dart';
import 'package:wowit/widgets/base_widget.dart';
import 'package:wowit/widgets/filter_widget/util.dart';
import 'component/filter_screen.dart';
import 'component/seached_list.dart';
import 'component/search_screen.dart';

typedef OnMapSortationValueChanged = void Function(
  String fieldName,
  ESortMode eSortMode,
);
typedef OnValueChanged = void Function(
  String fieldName,
  Object value,
);

typedef SortationValueGetter = ESortMode Function(String fieldName);
typedef ConditionValueGetter = Object Function(String fieldName);

typedef DataLoader = Future<Map<String, List<Map<String, Object>>>> Function();

typedef GetConditionView = Widget Function(
  Map<String, Object?> mapCondition,
  OnValueChanged onValueChanged,
);

typedef GetLineView = Widget Function(Map<String, Object> dataItem);

abstract class FilterWidget<T extends BaseController> extends BaseWidget<T> {
  FilterWidget(
    super.baseController, {
    super.key,
    super.tag,
    super.isComponent,
    this.fieldNameSearching,
    this.title,
    this.color,
    this.searchedListPadding,
    this.filterIcon,
    this.isFilterOnServer = false,
    required List<FilterItem> filterItems,
  })  : filterController = FilterController(filterItems),
        loadSearchedData = (baseController is SearchedController)
            ? (baseController as SearchedController).loadSearchedData
            : null;

  final bool isFilterOnServer;
  final String? title;
  final Color? color;
  final FilterController filterController;
  final String? fieldNameSearching;
  final EdgeInsets? searchedListPadding;
  final Widget? filterIcon;
  final SearchedDataLoader? loadSearchedData;

  void loadMore() {
    final baseController = getController(tag: tag);
    if (baseController is LoadMoreBaseController) {
      baseController.loadMore();
    } else {
      if (kDebugMode) {
        print('Please extends LoadMoreController');
      }
    }
  }

  Widget getConditionView(
    Map<String, Object?> mapCondition,
    void Function(String fieldName, Object value) onValueChange,
  );

  Widget getLineView(Map<String, Object> dataItem) {
    if (kDebugMode) {
      print('FilterWidget: Need to provide a LineView');
    }
    return const Text('Need to provide a LineView (Override getLineView())');
  }

  void _onApply() {
    if (isFilterOnServer) {
      applyFilterOnServer();
    } else {
      applyFilterBaseOnList();
    }
    Get.back();
  }

  void onSortationValueChanged(String fieldName, ESortMode sortMode) {
    filterController.updateMapSortation(fieldName, sortMode);
  }

  void onConditionValueChanged(String? fieldName, Object value) {
    filterController.updateMapCondition(fieldName, value);
  }

  void goToFilterView() {
    Get.to(
      () => FilterScreen(
        getConditionView: getConditionView,
        filterController: filterController,
        title: title,
        onApply: _onApply,
        onClear: _onClear,
        color: color,
      ),
    );
  }

  void goToSearchView() {
    if (kDebugMode) {
      print('loadSearchedData: $loadSearchedData');
    }
    Get.to(
      () => SearchScreen(
        searchedListModel: SearchedListModel(
          getConditionView: getConditionView,
          filterItems: filterController.filterItems,
          getLineView: getLineView,
          title: title ?? '',
          searchedListPadding: searchedListPadding ?? const EdgeInsets.all(0),
          isFilterOnServer: isFilterOnServer,
        ),
        loadSearchedData: loadSearchedData,
        fieldNameSearching: fieldNameSearching ?? 'title',
        filterIcon: filterIcon ?? const SizedBox.shrink(),
        inputDecoration: inputDecoration(),
      ),
    );
  }

  InputDecoration inputDecoration(){
    return InputDecoration(
      border: InputBorder.none,
      hintText: Message.getMessage('HOME_Search_Tim_Kiem'),
    );
  }
  

  // Lọc dựa trên data đã load về.
  Future<void> applyFilterBaseOnList() async {
    if (kDebugMode) {
      print('${applyFilterBaseOnList.runtimeType} applySort');
    }
    final baseController = getController(tag: tag);

    final loadedData = await baseController.loadData();
    if (loadedData == null) return;

    final filterDataFieldName = getFilterFieldName(loadedData);

    if (filterDataFieldName != null &&
        loadedData[filterDataFieldName] != null) {
      final filteredData =
          filterController.applyFilter(loadedData[filterDataFieldName]);
      loadedData[filterDataFieldName] = filteredData;
      baseController.setData(loadedData);
      baseController.update();
    }
  }

  // Sắp xếp dựa trên data đã load về.
  void applySortBaseOnList() {
    final baseController = getController(tag: tag);
    if (kDebugMode) {
      print('${baseController.runtimeType} applySort');
    }

    final loadedData = baseController.getData();

    final filterDataFieldName = getFilterFieldName(loadedData);

    if (filterDataFieldName != null &&
        loadedData[filterDataFieldName] != null) {
      final filteredData =
          filterController.applySort(loadedData[filterDataFieldName]!);

      loadedData[filterDataFieldName] = filteredData;
    }

    baseController.setData(loadedData);
    baseController.update();
  }

  // Sắp xếp dựa trên procedure trên server
  void applySortOnServer() {
    final mapSortation = filterController.mapSortation;
    final fieldName = mapSortation.keys.first;
    final sortMode = mapSortation[fieldName];

    final controller = getController(tag: tag);

    if (controller is LoadMoreBaseController) {
      controller.sort(fieldName, sortMode == ESortMode.smallestToLargest);
    } else {
      if (kDebugMode) {
        print('Please extends LoadMoreController');
      }
    }
  }

  // Lọc dựa trên procedure trên server
  void applyFilterOnServer() {
    final controller = getController(tag: tag);
    if (controller is LoadMoreBaseController) {
      controller.filter(filterController.filterMapCondition);
    } else {
      if (kDebugMode) {
        print('Please extends LoadMoreController');
      }
    }
  }

  Widget getLoadingView() {
    return const Padding(
      padding: EdgeInsets.only(top: 10, bottom: 40),
      child: Center(
        child: CircularProgressIndicator(),
      ),
    );
  }

  Widget getEndOfPageView() {
    return Container(
      padding: const EdgeInsets.only(top: 16, bottom: 16),
      decoration: const BoxDecoration(
        color: Color(0xff2a367e),
        borderRadius: BorderRadius.all(Radius.circular(10)),
      ),
      child: Center(
        child: Text(
          Message.getMessage('FilterWidget_ban_da_den_cuoi_trang'),
          style: const TextStyle(
            color: Colors.white,
            fontWeight: FontWeight.bold,
          ),
        ),
      ),
    );
  }

  void _onClear() {
    filterController.clear();
  }

  Object? getValueConditionByFieldName(String fieldName) {
    return filterController.mapCondition[fieldName];
  }

  ESortMode? getValueSortationByFieldName(String fieldName) {
    return filterController.mapSortation[fieldName];
  }
}

class FilterItem {
  const FilterItem({
    required this.fieldName,
    required this.filterMode,
    this.maxValue,
    this.defaultValue,
    this.allValue,
  });

  final String fieldName;

  final EFilterMode filterMode;

  /// Gia tri mac dinh
  /// Nen cung cap gia tri mac dinh khi [filterMode] == [EFilterMode.range]
  final Object? defaultValue;

  // Gia tri khi chon 'tat ca'
  // Gap gia tri nay thi se in ra toan bo list ma khong filter.
  final Object? allValue;

  // Gia tri max khi EFilterMode = inRangeOrGreaterThanMax
  final num? maxValue;
}

enum EFilterMode {
  equal,
  range,
  contain,
  lessThan,
  lessOrEqual,
  greaterThan,
  greaterOrEqual,
  inRangeOrGreaterThanMax,
  insensitiveAndUnsignedContain
}

enum ESortMode {
  smallestToLargest,
  largestToSmallest,
}
