import 'dart:io';
import 'package:flutter/material.dart';
import 'package:wowit/image_url.dart';
import 'package:wowit/methods.dart';
import 'package:wowit/widgets/wow_cache_manager/wow_cache_manager.dart';

/// Icon cho app, có thể dùng [IconData], [int] id ảnh, [String] tên ảnh
///
/// Nếu lấy ảnh trên server thì sẽ được lưu xuống cache
///
/// Ví dụ:
///```dart
/// WowIcon(12);
/// WowIcon('logo_8xland.png'),
/// WowIcon(Icons.add),
///```
class WowIcon extends StatefulWidget {
  const WowIcon(
    this.value, {
    super.key,
    this.size,
    this.boxFit = BoxFit.scaleDown,
    this.color,
    this.loadingWidget,
    this.errorWidget,
    this.scale = 1.0,
    this.semanticLabel,
    this.excludeFromSemantics = false,
    this.opacity,
    this.colorBlendMode,
    this.alignment = Alignment.center,
    this.repeat = ImageRepeat.noRepeat,
    this.centerSlice,
    this.matchTextDirection = false,
    this.gaplessPlayback = false,
    this.isAntiAlias = false,
    this.filterQuality = FilterQuality.low,
    this.cacheWidth,
    this.cacheHeight,
    this.waitingColor,
  })  : height = size,
        width = size;

  const WowIcon.image(
    this.value, {
    super.key,
    this.height,
    this.width,
    this.boxFit = BoxFit.scaleDown,
    this.color,
    this.loadingWidget,
    this.errorWidget,
    this.scale = 1.0,
    this.semanticLabel,
    this.excludeFromSemantics = false,
    this.opacity,
    this.colorBlendMode,
    this.alignment = Alignment.center,
    this.repeat = ImageRepeat.noRepeat,
    this.centerSlice,
    this.matchTextDirection = false,
    this.gaplessPlayback = false,
    this.isAntiAlias = false,
    this.filterQuality = FilterQuality.low,
    this.cacheWidth,
    this.cacheHeight,
    this.waitingColor,
  }) : size = (height ?? 0) > (width ?? 0) ? height : width;

  /// [value] có thể là id ảnh (ID) hoặc tên ảnh (String) hoặc là IconData
  ///
  /// Ví dụ:
  /// ```dart
  /// WowIcon(12)
  /// WowIcon('logo_8xland.png')
  /// WowIcon(Icons.add),
  /// ```
  final dynamic value;

  /// Kích thước icon
  final double? size;

  /// Chiều cao
  final double? height;

  /// Chiều dài
  final double? width;

  /// Hiển thị widget khi icon lỗi
  final Widget? errorWidget;

  /// Hiển thị widget khi đang load icon
  final Widget? loadingWidget;

  final BoxFit? boxFit;
  final Color? color;
  final double scale;
  final String? semanticLabel;
  final bool excludeFromSemantics;
  final Animation<double>? opacity;
  final BlendMode? colorBlendMode;
  final AlignmentGeometry alignment;
  final ImageRepeat repeat;
  final Rect? centerSlice;
  final bool matchTextDirection;
  final bool gaplessPlayback;
  final bool isAntiAlias;
  final FilterQuality filterQuality;
  final int? cacheWidth;
  final int? cacheHeight;
  final Color? waitingColor;

  @override
  State<WowIcon> createState() => _WowIconState();
}

class _WowIconState extends State<WowIcon> {
  late String url;
  File? file;

  @override
  void initState() {
    super.initState();
    _getUrl();
    _getImage();
  }

  @override
  void didUpdateWidget(covariant WowIcon oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.value != widget.value) {
      _getUrl();
      _getImage();
    }
  }

  void _getUrl() {
    url = Methods.getURL(
      ImageByID.getUrlImage(widget.value, defaultUrl: ''),
    );
  }

  void _getImage() async {
    if (url.isNotEmpty) {
      File image = await WowCacheManager.getImage(url);
      if (mounted) {
        setState(() {
          file = image;
        });
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    if (widget.value is IconData) {
      return _icon();
    }
    return _image();
  }

  Widget _icon() {
    return Icon(
      widget.value,
      size: widget.size,
      color: widget.color,
      semanticLabel: widget.semanticLabel,
    );
  }

  Widget _image() {
    if (file == null) {
      return widget.loadingWidget ?? _waitingView();
    }
    return Image.file(
      file!,
      width: widget.width,
      height: widget.height,
      key: widget.key,
      fit: widget.boxFit,
      color: widget.color,
      scale: widget.scale,
      errorBuilder: (_, __, ___) => widget.errorWidget ?? _waitingView(),
      semanticLabel: widget.semanticLabel,
      excludeFromSemantics: widget.excludeFromSemantics,
      opacity: widget.opacity,
      colorBlendMode: widget.colorBlendMode,
      alignment: widget.alignment,
      repeat: widget.repeat,
      centerSlice: widget.centerSlice,
      matchTextDirection: widget.matchTextDirection,
      gaplessPlayback: widget.gaplessPlayback,
      isAntiAlias: widget.isAntiAlias,
      filterQuality: widget.filterQuality,
      cacheWidth: widget.cacheWidth,
      cacheHeight: widget.cacheHeight,
    );
  }

  Widget _waitingView() {
    return Container(
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(10),
        color: widget.waitingColor ?? const Color(0xFFF5F5F5),
      ),
      width: widget.width,
      height: widget.height,
    );
  }
}
